#include <LiquidCrystal.h>
//#include "LiquidCrystal_1602_RUS.h"
#include <EEPROM.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <HX711.h>


// А0 кнопки
// А1 SCK весы
// А2 DT весы
// D2 датчик t
// D12 пищалка
// D13 реле подачи воды в дэф

LiquidCrystal lcd(8, 9, 4, 5, 6, 7);
//LiquidCrystal lcd(44, 42, 52, 50, 48, 46);

int Relay = 13;// D13 реле
int Zumer = 12;// D12 пищалка
HX711 scale(A2, A1);// инициализация весов. Указываем в какие пины подключен АЦП hx711 (DT(DOUT)  - pin A2, SCK - pin A1)
OneWire  oneWire(2); // датчик температуры подключен к выводу D2
DallasTemperature ds(&oneWire);
DeviceAddress sensor1 = {0x28, 0xFF, 0x81, 0xC5, 0x92, 0x16, 0x04, 0xB6};//царга
DeviceAddress sensor2 = {0x28, 0xFF, 0xC8, 0x20, 0xC0, 0x16, 0x05, 0x0E};//дэф
DeviceAddress sensor3 = {0x28, 0xFF, 0x74, 0x0F, 0xC0, 0x16, 0x05, 0x8A};//куб

unsigned long otsutDatT = 0;//таймер отсутствия сигнала от датчика температуры
unsigned long previousMillis = 0; // таймер для борьбы с дребезгом кнопок
unsigned long TimeTrevoga = 0;//таймер отсутствия сигнала от датчика температуры
unsigned long vesMillis = 0; // таймер для весов
unsigned long Tok_Millis = 0; // таймер работает при режиме разгон
int TSensorOff = 0;
int Koef_Sen_Off=0;
float tempSensor[3];  // массив с показаниям датчика t
float tempDelta = 0; // массив для установки дельты t
byte KEY; // значение нажатой кнопки
int button = 0; // флаг нажатия кнопки
byte button_Long = 0;// флаг долгого нажатия кнопки
int vestelo; // максимальный вес
float vesves;//значение веса на весах
byte menuLevel = 0;
int realI = 0; // получаемое от ардуино МИНИ значение реального тока
int fakt_I = 0;// полученное значение тока *11
int tok = 0; // ток  деленное на 11
int Wtrazgon; // мощность разгона хранится в EEPROM
int WtOtbor;
int razgon = 0; //флаг режима разгон



void readSet() { // считываем EEPROM по сохраненной температуре и мощности
  EEPROM.get(0, tempDelta);
  EEPROM.get(5, vestelo);
  EEPROM.get(10, Wtrazgon);
  EEPROM.get(15, WtOtbor);
}

void getTemp() { // читаем температуру с датчиков и заполняем массив
  ds.requestTemperatures();
  tempSensor[0] = ds.getTempC(sensor1);
  tempSensor[1] = ds.getTempC(sensor2);
  tempSensor[2] = ds.getTempC(sensor3);
}

byte Key() { // проверяем кнопки шилда
  if (analogRead(0) < 50) return 3; // вправо
  else if (analogRead(0) < 180) return 2; //вверх
  else if (analogRead(0) < 350) return 4;// вниз
  else if (analogRead(0) < 550) return 1; // влево
  else if (analogRead(0) < 800) return 5; //sekect
  else if (analogRead(0) <= 1100) return 0;
}

void setup() {
  Serial.begin(9600);
//  Serial1.begin(9600);
//  Serial2.begin(9600);
//  Serial3.begin(9600);

  ds.begin(); //включаем датчики t
  ds.setResolution(sensor1, 12); // определяем разрешение датчика температуры
  ds.setResolution(sensor2, 12);
  ds.setResolution(sensor3, 11);

  lcd.begin(16, 2);  // включаем экран
  lcd.clear();       // очищаем экран

  pinMode(Relay, OUTPUT);// активируем выход пина реле
  pinMode(Zumer, OUTPUT);// активируем выход пина пищалки

  digitalWrite(Relay, LOW);  // пин пока выключен
  digitalWrite(Zumer, LOW);  // пин пока выключен

  scale.set_scale(385.65); // Устанавливаем для весов калибровочный коэффициент

  readSet();  // читаем настройки из еепром
  if (isnan(tempDelta)) {
    tempDelta = 70;
    EEPROM.put(0, tempDelta);
  }
  if (isnan(vestelo)) {
    vestelo = 0;
    EEPROM.put(5, vestelo);
  }
  if (isnan(Wtrazgon)) {
    Wtrazgon = 1000;
    EEPROM.put(10, Wtrazgon);
  }
  if (isnan(WtOtbor)) {
    WtOtbor = 1000;
    EEPROM.put(15, WtOtbor);
  }
}

void loop() {
  //проверка клавиатуры
  //Serial.println(Key());
  //Serial.println(analogRead(0));
  //Serial.println("");
  
  Key();//убираем дребезг кнопок
  if (Key() != 0) {
    if (millis() - previousMillis >= 400) {
      previousMillis = millis();
      KEY = Key();
      button += 1;
    }  else KEY = 0;
    if (button > 10) button_Long = 1;// фключаем флаг долгого нажатия
  }
  else {
    KEY = 0;//
    button = 0;
    button_Long = 0;
  }

  readSet();  // читаем настройки из еепром

  if (Key() == 0) getTemp();// считываем температуру, только если не нажата кнопка


  if (millis() - vesMillis >= 2000 && Key() == 0) { //работаем с весами, только если не нажата кнопка
    vesMillis = millis();
    scale.power_up();                              // Отключаем энергосберегающий режим hx711
    vesves = scale.get_units(3);                   // Запоминаем текущий вес
    scale.power_down();                            // отключаем весы
  }

  if (tempSensor[0] <= 0 || tempSensor[1] <= 0 || tempSensor[2] <= 0) {// включим флаг тревоги если не работает датчик температуры
    if (tempSensor[0] < 0) tempSensor[0] = 0;
    if (tempSensor[1] < 0) tempSensor[1] = 0;
    if (tempSensor[2] < 0) tempSensor[2] = 0;
    if (millis() - otsutDatT >= 5000) {
      otsutDatT = millis();
      Koef_Sen_Off += 1;
      if (Koef_Sen_Off >=2) TSensorOff = 1;
    }
  }
  else {
    TSensorOff = 0;
    Koef_Sen_Off = 0;
  }


  // получаем от мини фактическое значение тока в настоящий момент
  if (Serial.available() > 0) {// определяем значение реальной мощности
    realI = Serial.read();
    fakt_I = realI * 11;
  }

  if (tempSensor[1] >= 80 || tempSensor[2] > 99 || vesves >= vestelo || TSensorOff == 1) {

    if (millis() - TimeTrevoga >= 1000) {
      TimeTrevoga = millis();
      tone(Zumer, 2000, 300); // включаем пищалку
      tok = 0 ; // вычисляем значение тока с округлением
      Serial.write(tok); // передаем значение на ардумини
    }
    razgon = 2;
    digitalWrite(Relay, HIGH);// включаем подачу воды в дэф

  }
  else
  {
    if (tempSensor[0] <= 70) {//включение режима разгона если t низ царги меньше 70
      if (millis() - Tok_Millis >= 1000) { // расчет мощности и отправка значения на МИНИ
        Tok_Millis = millis();
        tok = round(Wtrazgon / 11) ; // вычисляем значение тока с округлением
        Serial.write(tok); // передаем значение на ардумини
        // флаг режима разгон
      }
      razgon = 1;// флаг режима разгон
    }
    if (tempSensor[0] > 70) {
      if (millis() - Tok_Millis >= 1000) { // расчет мощности и отправка значения на МИНИ
        Tok_Millis = millis();
        tok = round(WtOtbor / 11) ; // вычисляем значение тока с округлением
        Serial.write(tok); // передаем значение на ардумини
      }
      razgon = 0;// снимим флаг режима разгон
    }
    // управляем реле подачи воды в дэф
    if (tempSensor[0] >= tempDelta && tempSensor[1] >= 70)digitalWrite(Relay, HIGH);
    if ( tempSensor[0] < tempDelta || tempSensor[1] < 70) digitalWrite(Relay, LOW);
    noTone(Zumer);// выключаем пищалку
  }



  if (menuLevel == 0)
  {
    lcd.setCursor(0, 0);
    lcd.print (tempSensor[0], 1);  // показываем температуру с датчика царги
    lcd.setCursor(5, 0);
    lcd.print (tempSensor[1], 1);  // показываем температуру с датчика дефа
    lcd.setCursor(10, 0);
    lcd.print (tempSensor[2], 1);  // показываем температуру с датчика куба
    lcd.print ("  ");
    lcd.setCursor(0, 1);
    lcd.print(tempDelta, 1); // показываем установленную температуру дельты
    lcd.print (" ");

    long time = millis() / 1000;
    lcd.setCursor(6, 1);
    if ((time / 60 / 60) % 60 < 10) {
      lcd.print ("0");
    }
    lcd.print (time / 60 / 60); // час
    lcd.setCursor(8, 1);
    if (time % 2 == 1)  lcd.print (":");
    else   lcd.print (" ");
    lcd.setCursor(9, 1);
    if (time / 60 % 60 < 10) {
      lcd.print ("0");
    }
    lcd.print ((time / 60) % 60);
    lcd.setCursor(12, 1);
    if (digitalRead(Relay) == HIGH) lcd.print ("Klap");
    else {
      if (razgon == 1) lcd.print ("Razg");
      if (razgon == 0) lcd.print ("Telo");
      if (razgon == 2) lcd.print ("Trev");
    }

//    lcd.setCursor(0, 2);
//    //  lcd.print(L"Вес:");
//    lcd.print("Ves:");
//    lcd.print (vestelo);
//    lcd.print ("/");
//    lcd.print (vesves);
//    lcd.print ("    ");
//
//    lcd.setCursor(0, 3);
//    if (razgon == 2) lcd.print (tok);
//    if (razgon == 1) lcd.print (Wtrazgon);
//    if (razgon == 0) lcd.print (WtOtbor);
//    lcd.print ("Wt");
//    lcd.print ("/");
//    lcd.print (fakt_I);
//    lcd.print ("Wt");
//    lcd.print ("    ");


    if (KEY == 4) { // если нажата кнопка
      if (button_Long == 0) tempDelta -= 0.1; // изменяем значение
      if (button_Long == 1) tempDelta -= 1; // изменяем значение
      if (tempDelta < 70) tempDelta = 100;
      if (tempDelta > 100) tempDelta = 70;
      EEPROM.put(0, tempDelta); // сохраняем в еепром
    }

    if (KEY == 2) {
      if (button_Long == 0) tempDelta += 0.1; // изменяем значение
      if (button_Long == 1) tempDelta += 1;
      if (tempDelta > 100) tempDelta = 70;
      if (tempDelta < 70) tempDelta = 100;
      EEPROM.put(0, tempDelta);
    }

    if (KEY == 3) { //  шагаем следующее меню
      menuLevel = 1;
      lcd.clear();
      KEY = 0;
    }
  }

  if (menuLevel == 1)// задаем значение для весов
  {
    lcd.setCursor(0, 0);
    lcd.print ("Fakt ves: ");  // показываем температуру с датчика царги
    lcd.print (vesves);  // показываем температуру с датчика дефа
    lcd.print ("   ");
    lcd.setCursor(0, 1); // установим таймер времени
    lcd.print ("Max  ves: ");  // показываем температуру с датчика царги
    lcd.print (vestelo);  // показываем температуру с датчика дефа
    lcd.print ("   ");

    if (KEY == 4) { // если нажата кнопка
      if (button_Long == 0) vestelo -= 10; // изменяем значение
      if (button_Long == 1) vestelo -= 100; // изменяем значение
      if (vestelo < 0) vestelo = 4000;
      if (vestelo > 4000) vestelo = 0;
      EEPROM.put(5, vestelo); // сохраняем в еепром
    }

    if (KEY == 2) {
      if (button_Long == 0) vestelo += 10; // изменяем значение
      if (button_Long == 1) vestelo += 100; // изменяем значение
      if (vestelo < 0) vestelo = 4000;
      if (vestelo > 4000) vestelo = 0;
      EEPROM.put(5, vestelo); // сохраняем в еепром
    }

    if (KEY == 3) { //  шагаем следующее меню
      menuLevel = 2;
      lcd.clear();
      KEY = 0;
    }
    if (KEY == 1) { //  шагаем предидущее меню
      menuLevel = 0;
      lcd.clear();
      KEY = 0;
    }
  }
  if (menuLevel == 2)// задаем мощность тена
  {
    lcd.setCursor(0, 0);
    lcd.print ("Real tok: ");  // показываем температуру с датчика царги
    lcd.print (fakt_I);  // показываем температуру с датчика дефа
    lcd.print ("Wt  ");

    lcd.setCursor(0, 1); // установим таймер времени
    lcd.print ("Tok telo: ");  // показываем температуру с датчика царги
    lcd.print (WtOtbor);  // показываем температуру с датчика дефа
    lcd.print ("Wt  ");

    if (KEY == 4) { // если нажата кнопка
      if (button_Long == 0) WtOtbor -= 10; // изменяем значение
      if (button_Long == 1) WtOtbor -= 100; // изменяем значение
      if (WtOtbor < 0) WtOtbor = 2500;
      if (WtOtbor > 2500) WtOtbor = 0;
      EEPROM.put(15, WtOtbor); // сохраняем в еепром
    }

    if (KEY == 2) {
      if (button_Long == 0) WtOtbor += 10; // изменяем значение
      if (button_Long == 1) WtOtbor += 100; // изменяем значение
      if (WtOtbor < 0) WtOtbor = 2500;
      if (WtOtbor > 2500) WtOtbor = 0;
      EEPROM.put(15, WtOtbor); // сохраняем в еепром
    }

    if (KEY == 3) { //  шагаем следующее меню
      menuLevel = 3;
      lcd.clear();
      KEY = 0;
    }
    if (KEY == 1) { //  шагаем следующее меню
      menuLevel = 1;
      lcd.clear();
      KEY = 0;
    }
  }
  if (menuLevel == 3)// задаем максимальную мощность тена
  {
    lcd.setCursor(0, 0);
    lcd.print ("Real tok: ");  // показываем температуру с датчика царги
    lcd.print (fakt_I);  // показываем температуру с датчика дефа
    lcd.print ("Wt ");

    lcd.setCursor(0, 1); // установим таймер времени
    lcd.print ("Max  tok: ");  // показываем температуру с датчика царги
    lcd.print (Wtrazgon);  // показываем температуру с датчика дефа
    lcd.print ("Wt ");

    if (KEY == 4) { // если нажата кнопка
      if (button_Long == 0) Wtrazgon -= 10; // изменяем значение
      if (button_Long == 1) Wtrazgon -= 100; // изменяем значение
      if (Wtrazgon < 0) Wtrazgon = 2500;
      if (Wtrazgon > 2500) Wtrazgon = 0;
      EEPROM.put(10, Wtrazgon); // сохраняем в еепром
    }

    if (KEY == 2) {
      if (button_Long == 0) Wtrazgon += 10; // изменяем значение
      if (button_Long == 1) Wtrazgon += 100; // изменяем значение
      if (Wtrazgon < 0) Wtrazgon = 2500;
      if (Wtrazgon > 2500) Wtrazgon = 0;
      EEPROM.put(10, Wtrazgon); // сохраняем в еепром
    }

    //    if (KEY == 3) { //  шагаем следующее меню
    //    menuLevel = 4;
    //    lcd.clear();
    //    }
    if (KEY == 1) { //  шагаем следующее меню
      menuLevel = 2;
      lcd.clear();
      KEY = 0;
    }
  }

}

